using FileStorage.Repository.SqlSugar;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.StaticFiles;
using Microsoft.Extensions.Options;
using SqlSugar;
using XF.Domain.Authentications;

namespace FileStorage.Host.Controllers
{
    [Authorize]
    [ApiController]
    [Route("file")]
    public class FileController : ControllerBase
    {
        private readonly ILogger<FileController> _logger;
        private readonly IFileStorage _fileStorage;
        private readonly ISugarUnitOfWork<FileStorageDbContext> _uow;
        private readonly ISessionContext _sessionContext;

        public FileController(
            ILogger<FileController> logger,
            IFileStorage fileStorage,
            ISugarUnitOfWork<FileStorageDbContext> uow,
            ISessionContext sessionContext
            )
        {
            _logger = logger;
            _fileStorage = fileStorage;
            _uow = uow;
            _sessionContext = sessionContext;
        }

        [HttpPost("upload")]
        public IActionResult Upload(IFormFile fileData)
        {
            //var fileData = Request.Form.Files[0];
            var file = _fileStorage.Upload(fileData.FileName, fileData.Length, "", fileData.OpenReadStream(), _sessionContext.ClientId ?? string.Empty);
            return Ok(new { Id = file.Id, fileName = file.FileName });
        }

        [HttpGet("getfileurl/{id}")]
        public async Task<IActionResult> GetFileUrl(string id)
        {
            var uri = await _fileStorage.GetFileUrlAsync(id);
            return Ok(new { Uri = uri });
        }


        [HttpGet("downloadfile/{id}")]
        public async Task<IActionResult> DownLoadFile(string id)
        {
            var (stream, content, fileName) = _fileStorage.DownLoadFile(id);
            if (stream != null)
            {
                return File(stream, content, fileName);
            }
            return null;
        }

        [HttpGet("delfile/{id}")]
        public async Task<bool> DelFile(string id)
        {
            return await _fileStorage.DelFileAsync(id);
        }


        [HttpGet("createdb")]
        public Task CreateDb()
        {
            var db = _uow.Db;
            db.DbMaintenance.CreateDatabase();
            db.CodeFirst.InitTables<FileMetadata>();
            return Task.CompletedTask;
        }
    }
}